#include "GPL951_Body.h"
#include "I2C.h"

//***************************************************************************************
// Constant Definitioin Area
//***************************************************************************************
//24LC32 : 1 0 1 0 A2 A1 A0 R/W
#define DeviceAddress_Write		0xA0
#define DeviceAddress_Read		0xA1

//***************************************************************************************
// Function pre-announcement Area
//***************************************************************************************
extern Data[1000];
extern PageWriteSourceData[32];
extern I2C_Error_Flag;
//;==================================================================
//;Function Setup
//;==================================================================
//****************************************************************
// Function    : EEPROM_ByteWrite()
// Description : Byte Write
// Destroy     : 
// Parameter   : WordAddr, Data
// Return      : None
// Note        : None
//****************************************************************
unsigned EEPROM_ByteWrite(unsigned WordAddr, unsigned Data)
{
	unsigned Address;
	Address = WordAddr;
	
	// User need to check I2C_Error_Flag (0x80 : write error ; 0x40 : read error) 
	I2C_Start(DeviceAddress_Write);
	I2C_SendByte(Address>>8);
	I2C_SendByte(Address&0xFF);
	I2C_SendByte(Data);
	I2C_Stop();
		while (I2C_Error_Flag != 0)	// I2C Failed
			*P_WatchDog_Clear = C_WDT_Clear;
}

//****************************************************************
// Function    : EEPROM_PageWrite()
// Description : Page Write
// Destroy     : 
// Parameter   : WordAddr
// Return      : None
// Note        : None
//****************************************************************
unsigned EEPROM_PageWrite(unsigned WordAddr)
{
	unsigned Address;
	int i;
	
	// User need to check I2C_Error_Flag (0x80 : write error ; 0x40 : read error) 
	Address = WordAddr;
	I2C_Start(DeviceAddress_Write);
	I2C_SendByte(Address>>8);
	I2C_SendByte(Address&0xFF);
	
	for (i=0; i<32; i++)
		I2C_SendByte(PageWriteSourceData[i]);
	
	I2C_Stop();
	
	while (I2C_Error_Flag != 0)	// I2C Failed
		*P_WatchDog_Clear = C_WDT_Clear;
}

//****************************************************************
// Function    : EEPROM_RandomRead()
// Description : Current Address Read
// Destroy     : 
// Parameter   : WordAddr
// Return      : Data
// Note        : None
//****************************************************************
unsigned EEPROM_RandomRead(unsigned WordAddr)
{
	unsigned Address;
	unsigned Data;

	// User need to check I2C_Error_Flag (0x80 : write error ; 0x40 : read error) 
	Address = WordAddr;		
	I2C_Start(DeviceAddress_Write);
	I2C_SendByte(Address>>8);
	I2C_SendByte(Address&0xFF); 
	I2C_ReadStart(DeviceAddress_Read);
//	Data = I2C_ReadByte();
	I2C_ReadByte();
	Data = *P_I2C_Data;
	I2C_Stop(); 	
	
	while (I2C_Error_Flag != 0)	// I2C Failed
		*P_WatchDog_Clear = C_WDT_Clear;		
		
	return Data;
}

//****************************************************************
// Function    : EEPROM_SequentailRead()
// Description : Sequentail Read
// Destroy     : 
// Parameter   : WordAddr
// Return      : Data
// Note        : None
//****************************************************************
unsigned EEPROM_SequentialRead(unsigned WordAddr, unsigned Length)
{
	unsigned Address;
	unsigned Data_Length;	
	unsigned Temp_Data;
	int i;
	
	Address = WordAddr;	
	Data_Length = Length;

	// User need to check I2C_Error_Flag (0x80 : write error ; 0x40 : read error) 
	Address = WordAddr;		
	I2C_Start(DeviceAddress_Write);
	I2C_SendByte(Address>>8);
	I2C_SendByte(Address&0xFF); 
	I2C_ReadStart(DeviceAddress_Read);
	for (i=0;i<Data_Length;i++) {
//		Temp_Data = I2C_SequentialReadByte();
//		Data[i] = Temp_Data;
		I2C_SequentialReadByte();
		Data[i] = *P_I2C_Data;
	}
	*P_I2C_Ctrl &= ~C_I2C_ACK_Enable;	// NACK
	CheckComplete_R();
	I2C_Stop(); 		

	while (I2C_Error_Flag != 0)	// I2C Failed
		*P_WatchDog_Clear = C_WDT_Clear;	
}




