 //==================================================================================
// The information contained herein is the exclusive property of
// Generalplus Technology Co. And shall not be distributed, reproduced,
// or disclosed in whole in part without prior written permission.
//       (C) COPYRIGHT 2010   Generalplus TECHNOLOGY CO.                            
//                   ALL RIGHTS RESERVED
// The entire notice above must be reproduced on all authorized copies.
//==================================================================================
//==========================================================================
// Program Name: I2C.asm
// Applied body: GPL95100UA Series Body
// History version
// Rev #     Date       Who     Comments
// -----  -----------  ------   --------------------------------------------
// 1.0     2018/02/05  			Oringinal Version
// 1.1	   2019/06/19			Modify I2C Error function
//					   			Add I2C Read / Write Error Flag (I2C_Error_Flag)	0x80 : Write error ; 0x40 : Read error

//***************************************************************************************
// Header File Included Area
//***************************************************************************************
.include GPL951_Body.inc


//***************************************************************************************
// Function Call Publication Area
//***************************************************************************************
.public  _I2C_IF_Initial
.public F_I2C_IF_Initial

.public  _I2C_Start
.public F_I2C_Start

.public  _I2C_ReadStart
.public F_I2C_ReadStart

.public  _I2C_Stop
.public F_I2C_Stop

.public  _I2C_SendByte
.public F_I2C_SendByte

.public  _I2C_ReadByte
.public F_I2C_ReadByte

.public  _I2C_SequentialReadByte
.public F_I2C_SequentialReadByte

.public  _ProgrammingWait
.public F_ProgrammingWait

.public  _CheckComplete_R
.public  F_CheckComplete_R
//***************************************************************************************
// RAM Definition Area
//***************************************************************************************
.RAM
.define D_TimeOutCnt    1000
.define	C_SystemClock	96000000
.define IIC_clock		300000
.define IIC_CLK_Reg		(C_SystemClock/2/IIC_clock)-1

.external _I2C_Error_Flag
.define D_I2C_W_Error   0x80
.define	D_I2C_R_Error	0x40
//***************************************************************************************
// CODE Definition Area
//***************************************************************************************
.CODE

//****************************************************************
// Function    : F_I2C_IF_Initial
// Description : Set I2C Master TX, I2C Enable
// Destroy     : 
// Parameter   : None
// Return      : None
// Note        : None
//****************************************************************
_I2C_IF_Initial:    .proc
F_I2C_IF_Initial:
    push R1 to [SP];
    	
    // Enable IOC I2C I/F
    R1 = [P_IOC_Mux]
    R1 |= C_IOC_I2C_En
    [P_IOC_Mux] = R1
    
	R1 = C_I2C_ACK_Enable + C_I2C_INT_Flag
	[P_I2C_Ctrl] = R1	
	R1 = C_I2C_Mode_MasterTX + C_I2C_DataOut_Enable
	[P_I2C_Status] = R1
	
	R1=IIC_CLK_Reg&0x00FF
	[P_I2C_Clk] = R1
	R1 = IIC_CLK_Reg&0x0F00
	R1 = R1 lsr 4
	R1 = R1 lsr 4
	R1 |= [P_I2C_Ctrl]	
	[P_I2C_Ctrl] = R1
		
	R1 = 0x10
	[P_I2C_Debounce] = R1
	R1 = C_I2C_En
	[P_I2C_MISC] = R1

    pop R1 from [SP];
    retf;
    .endp

//****************************************************************
// Function    : F_I2C_Start
// Description : 
// Destroy     : 
// Parameter   : None
// Return      : None
// Note        : None
//****************************************************************
_I2C_Start:    .proc
F_I2C_Start:
  	R1 = SP + 3;
    R1 = [R1];
F_I2C_SendAddr:
    push R2, R3 to [SP];
      
	R2 = [P_I2C_Ctrl]
	[P_I2C_Ctrl] = R2
	[P_I2C_Data] = R1				//send device address

	R2= C_I2C_Mode_MasterTX + C_I2C_Start + C_I2C_DataOut_Enable
	[P_I2C_Status] = R2
	call	F_CheckComplete

    pop R2, R3 from [SP];
	retf;
	.endp
	
//****************************************************************
// Function    : F_I2C_ReadStart
// Description : 
// Destroy     : 
// Parameter   : None
// Return      : None
// Note        : None
//****************************************************************
_I2C_ReadStart:    .proc
F_I2C_ReadStart:
  	R1 = SP + 3;
    R1 = [R1];
F_I2C_SendReadAddr:
    push R2, R3 to [SP];
      
	[P_I2C_Data] = R1
	
	R2 = C_I2C_Mode_MasterRX + C_I2C_Start + C_I2C_DataOut_Enable	// Master RX and start cmd
	[P_I2C_Status] = R2
	call	F_CheckComplete	

    pop R2, R3 from [SP];
	retf;
	.endp	
	
//****************************************************************
// Function    : F_I2C_Stop
// Description : 
// Destroy     : 
// Parameter   : None
// Return      : None
// Note        : None
//****************************************************************
_I2C_Stop:    .proc
F_I2C_Stop:
    push R2 to [SP];
    
    R2 = C_I2C_Mode_MasterTX + C_I2C_DataOut_Enable				//stop
	[P_I2C_Status] = R2	
	call	F_Delay

    pop R2 from [SP];
	retf;
	.endp	

//****************************************************************
// Function    : F_I2C_SendByte
// Description : 
// Destroy     : 
// Parameter   : None
// Return      : None
// Note        : None
//****************************************************************	
_I2C_SendByte:    .proc
    R1 = SP + 3;
    R1 = [R1];
F_I2C_SendByte:
    push R2, R3 to [SP];
    
	[P_I2C_Data] = R1;
	R1 = [P_I2C_Ctrl];
	[P_I2C_Ctrl] = R1;				//send data
	call	F_CheckComplete
    
    pop R2, R3 from [SP];
	retf;
	.endp
	
//****************************************************************
// Function    : F_I2C_ReadByte
// Description : 
// Destroy     : 
// Parameter   : None
// Return      : None
// Note        : None
//****************************************************************	
_I2C_ReadByte:    .proc
    R1 = SP + 3;
    R1 = [R1];
F_I2C_ReadByte:
    push R2, R3 to [SP];
	
	r2=[P_I2C_Ctrl]
	r2&=~C_I2C_ACK_Enable
	[P_I2C_Ctrl]=r2
	call	F_CheckComplete_R	
	
	R1 = [P_I2C_Data]
    
    pop R2, R3 from [SP];
	retf;
	
	.endp	
	
//****************************************************************
// Function    : F_I2C_SequentialReadByte
// Description : 
// Destroy     : 
// Parameter   : None
// Return      : None
// Note        : None
//****************************************************************	
_I2C_SequentialReadByte:    .proc
    R1 = SP + 3;
    R1 = [R1];
F_I2C_SequentialReadByte:
    push R2, R3 to [SP];
	
	r2=[P_I2C_Ctrl]
	r2|=C_I2C_ACK_Enable
	[P_I2C_Ctrl]=r2
	call	F_CheckComplete_R	
	
	R1 = [P_I2C_Data]
    
    pop R2, R3 from [SP];
	retf;
	
	.endp	
    
//****************************************************************
// Function    : F_CheckComplete
// Description : 
// Destroy     : 
// Parameter   : None
// Return      : None
// Note        : None
//****************************************************************	    
F_CheckComplete:
	push	R2 to [sp]
	
	R1 = D_TimeOutCnt
wait?:
	R1-=1
	jz	error?
	R2 = [P_I2C_Ctrl]
	test	R2,C_I2C_INT_Flag
	jnz _pass?
	jz	wait?
		
_pass?:		
	R2 = [P_I2C_Status]
	test	R2,C_I2C_Bus_Fail
	jnz	error?
	test	R2,C_I2C_NACK
	jnz	error?
//	call	F_Delay

	// ========== Frank add 2019.06.19 ==========
	// If write pass, clear write error flag
	R1 = [_I2C_Error_Flag]
	R1 &=~D_I2C_W_Error
	[_I2C_Error_Flag] = R1
	// ==========================================

	pop R2 from [sp]
	retf    
	
error?:			// User need to modify it for real product
	// ========== Frank add 2019.06.19 ==========
	// If write failed, set write error flag
	R1 = [_I2C_Error_Flag]
	R1 |= D_I2C_W_Error
	[_I2C_Error_Flag] = R1
	// ==========================================
	call 	_Reinitial?	
	pop		R2 from [sp]	
	retf	
	
_Reinitial?:	
// Solve I2C_SDA status = Low, can't pull high
	R1 = [P_I2C_MISC]
	R1 &= ~C_I2C_En
	[P_I2C_MISC] = R1
	
	NOP
	NOP
	
	R1 = [P_I2C_MISC]
	R1 |= C_I2C_En
	[P_I2C_MISC] = R1	
//////////////////////////////////////////////	
	call	F_I2C_IF_Initial
	retf
	
//****************************************************************
// Function    : F_CheckCompleteR
// Description : 
// Destroy     : 
// Parameter   : None
// Return      : None
// Note        : None
//****************************************************************	
_CheckComplete_R:
F_CheckComplete_R:
	push	r1, r2 to [sp]
	R1 = D_TimeOutCnt	
wait?:
	R1-=1
	jz	error?
	R2 = [P_I2C_Ctrl]
	test	R2,C_I2C_INT_Flag
	jnz _pass?	
	jz	wait?
		
_pass?:	
	R2 = [P_I2C_Status]
	test	R2,C_I2C_Bus_Fail
	jnz	error?
//	call	F_Delay

	// ========== Frank add 2019.06.19 ==========
	// If read pass, clear read error flag
	R1 = [_I2C_Error_Flag]
	R1 &=~D_I2C_R_Error
	[_I2C_Error_Flag] = R1
	// ==========================================

	pop r1, r2 from [sp]
	retf	
	
error?:					// User need to modify it for real product
	// ========== Frank add 2019.06.19 ==========
	// If read failed, set read error flag
	R1 = [_I2C_Error_Flag]
	R1 |= D_I2C_R_Error
	[_I2C_Error_Flag] = R1
	// ==========================================
	call 	_Reinitial?
	pop		r1, r2 from [sp]	
	retf
	
_Reinitial?:
// Solve I2C_SDA status = Low, can't pull high
	R1 = [P_I2C_MISC]
	R1 &= ~C_I2C_En
	[P_I2C_MISC] = R1
	
	NOP
	NOP
	
	R1 = [P_I2C_MISC]
	R1 |= C_I2C_En
	[P_I2C_MISC] = R1	
//////////////////////////////////////////////	
	call	F_I2C_IF_Initial
	retf			

//****************************************************************
// Function    : F_Delay
// Description : 
// Destroy     : 
// Parameter   : None
// Return      : None
// Note        : None
//****************************************************************		
//******************************
F_Delay:
	push r1 to [sp]
	R1 = 48
d1?:	
	R1-=1
	jnz	d1?
	pop	R1 from [sp]
	retf
	
//****************************************************************
// Function    : ProgrammingWait
// Description : 
// Destroy     : 
// Parameter   : None
// Return      : None
// Note        : None
//****************************************************************		
_ProgrammingWait:
F_ProgrammingWait:
	push	r1,r2 to [sp]
	r1=0x100
d1?:	
	r2=0x200
d2?:
	r2-=1
	jnz	d2?
	r1-=1
	jnz	d1?
	pop	r1,r2 from [sp]
	retf			